@@ -5,8 +5,10 @@ getWeights <- function(mf) {
   if (any(wts <= 0))
     stop(gettextf("non-positive weights are not allowed"),
          call.=FALSE)
-### FIXME: Wouldn't it be usefull to just remove any observations with
-### zero weights?
+### NOTE: We do not remove observations where weights == 0, because
+### that could be a somewhat surprising behaviour. It would also
+### require that the model.frame be evaluated all over again to get
+### the right response vector with the right number of levels.
   if(length(wts) && length(wts) != n)
     stop(gettextf("number of weights is %d should equal %d (number of observations)",
                   length(wts), n), call.=FALSE)
@@ -83,10 +85,15 @@ drop.coef2 <- function(X, tol = 1e-7, silent = FALSE, test.ans = FALSE)
 }
 
 
-drop.cols <- function(mf, silent = FALSE) {
-### drop columns from X and possibly NOM to ensure full column rank 
-### mf - list with X and possibly NOM design matrices.
-
+drop.cols <- function(mf, silent = FALSE) 
+### drop columns from X and possibly NOM and S to ensure full column
+### rank.
+### mf - list with X and possibly NOM and S design matrices. Includes
+### a ths object containing ths$alpha.names
+### 
+### returns: updated version of mf.
+{
+  nalpha <- length(mf$ths$alpha.names)
   ## X is assumed to contain an intercept at this point:
   Xint <- match("(Intercept)", colnames(mf$X), nomatch = 0)
     if(Xint <= 0) {
@@ -94,17 +101,23 @@ drop.cols <- function(mf, silent = FALSE) {
     warning("an intercept is needed and assumed")
   } ## intercept in X is guaranteed.
   if(!is.null(mf$NOM)){
+    ## store coef names:
+    mf$coef.names <- list()
+    mf$coef.names$alpha <-
+      paste(rep(mf$ths$alpha.names, ncol(mf$NOM)), ".",
+            rep(colnames(mf$NOM), each=nalpha), sep="")
+    mf$coef.names$beta <- colnames(mf$X)[-1]
     ## drop columns from NOM:
-    mf$coef.names <- list(alpha = colnames(mf$NOM),
-                          beta = colnames(mf$X)[-1])
     mf$NOM <- drop.coef2(mf$NOM, silent=silent)
-    mf$aliased <- list(alpha = attr(mf$NOM, "aliased"))
     ## drop columns from X:
     NOMX <- drop.coef2(cbind(mf$NOM, mf$X[,-1, drop=FALSE]),
                       silent=silent) 
     ## extract and store X:
     mf$X <- cbind("(Intercept)" = rep(1, nrow(mf$X)),
                   NOMX[,-seq_len(ncol(mf$NOM)), drop=FALSE])
+    ## store alias information:
+    mf$aliased <- list(alpha = rep(attr(mf$NOM, "aliased"),
+                         each=nalpha)) 
     mf$aliased$beta <- attr(NOMX, "aliased")[-seq_len(ncol(mf$NOM))]
     if(!is.null(mf$S)) {
       mf$coef.names$zeta <- colnames(mf$S)[-1]
@@ -118,10 +131,12 @@ drop.cols <- function(mf, silent = FALSE) {
     }
     return(mf)
   }
-  ## drop columns from X asssuming an intercept:
-  mf$coef.names <- list(beta = colnames(mf$X)[-1])
+  ## drop columns from X assuming an intercept:
+  mf$coef.names <- list(alpha = mf$ths$alpha.names,
+                        beta = colnames(mf$X)[-1])
   mf$X <- drop.coef2(mf$X, silent=silent)
-  mf$aliased <- list(beta = attr(mf$X, "aliased")[-1])
+  mf$aliased <- list(alpha = rep(0, nalpha),
+                     beta = attr(mf$X, "aliased")[-1])
   ## drop columns from S if relevant:
   if(!is.null(mf$S)) { 
     Sint <- match("(Intercept)", colnames(mf$S), nomatch = 0)
@@ -328,7 +343,8 @@ eclm.start <- function(y, threshold, X, NOM=NULL, S=NULL,
 clmm.start <- function(frames, link, threshold) {
   ## get starting values from clm:
   fit <- with(frames,
-              clm.fit(y, X, wts, off, link=link, threshold=threshold))
+              clm.fit(y=y, X=X, weights=wts, offset=off, link=link,
+                      threshold=threshold)) 
   
   ## initialize variance parameters to zero:
   start <- c(fit$par, rep(0, length(frames$grList)))
@@ -374,19 +390,22 @@ set.start <-
                  NOM=frames$NOM, has.intercept=TRUE)
     if(NCOL(frames$S) > 1 || link == "cauchit") {
 ### NOTE: only special start if NCOL(frames$S) > 1 (no reason for
-### special start if scale is only offset and no predictors.
+### special start if scale is only offset and no predictors).
+### NOTE: start cauchit models at the probit estimates if start is not
+### supplied: 
       rho$par <- start
       if(link == "cauchit") setLinks(rho, link="probit")
       else setLinks(rho, link)
       tempk <- rho$k
       rho$k <- 0
       ## increased gradTol:
-      fit <- clm.fit.env(rho, control=list(gradTol=1e-3)) 
+      fit <- try(clm.fit.env(rho, control=list(gradTol=1e-3)),
+                 silent=TRUE) 
       if(class(fit) == "try-error") 
         stop("Failed to find suitable starting values: please supply some",
              call.=FALSE)
-      start.iter <- fit$niter
       start <- c(fit$par, rep(0, NCOL(frames$S) - 1))
+      attr(start, "start.iter") <- fit$niter
       rho$k <- tempk
     }
   }
@@ -397,8 +416,6 @@ set.start <-
     stop(gettextf("length of start is %d should equal %d",
                   length(start), length.start), call.=FALSE)
 
-  ## start cauchit models at the probit estimates if start is not
-  ## supplied: 
   return(start)
 }
 
